'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.grantHaveAttributes = exports.grantHaveResources = exports.grantHaveMembers = exports.Grant = undefined;

var _immutable = require('immutable');

var _utils = require('../utils');

var _transformers = require('../transformers');

var Grant = exports.Grant = (0, _immutable.Record)({
  members: (0, _immutable.Set)(),
  resources: (0, _immutable.Set)(),
  attributes: (0, _immutable.Set)()
}, 'Grant');


Grant.lift = function (members, resources, attributes) {
  return new Grant({
    members: (0, _transformers.membersToSet)(members),
    resources: (0, _transformers.membersToSet)(resources),
    attributes: (0, _transformers.membersToSet)(attributes)
  });
};

Grant.isEmpty = function (grant) {
  return grant.members.isEmpty() || grant.resources.isEmpty() || grant.attributes.isEmpty();
};

var grantHaveMembers = exports.grantHaveMembers = (0, _utils.compose)(function (members) {
  return function (grant) {
    return !grant.members.intersect(members).isEmpty();
  };
}, _transformers.membersToSet);

var grantHaveResources = exports.grantHaveResources = (0, _utils.compose)(function (resources) {
  return function (grant) {
    return grant.resources.isSuperset(resources);
  };
}, _transformers.membersToSet);

var grantHaveAttributes = exports.grantHaveAttributes = (0, _utils.compose)(function (attributes) {
  return function (grant) {
    return !attributes || !grant.attributes.intersect(attributes).isEmpty();
  };
}, _transformers.membersToSet);