'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getOrCreateRole = exports.getRoleIfNotEmpty = exports.getRole = exports.getRoleName = exports.getRoleByName = exports.getRolesByMember = undefined;

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _utils = require('../utils');

var _transformers = require('../transformers');

var _role = require('./role');

var getRolesByMember = exports.getRolesByMember = function getRolesByMember(roles) {
  return (0, _utils.compose)(function (members) {
    return roles.filter(function (role) {
      return !role.members.intersect(members).isEmpty();
    });
  }, _transformers.membersToSet);
};

var getRoleByName = exports.getRoleByName = function getRoleByName(roles) {
  return function (roleName) {
    return roles.get(roleName, _role.Role.lift(roleName));
  };
};

var getRoleName = exports.getRoleName = function getRoleName(role) {
  return _role.Role.isRole(role) ? role.name : role;
};

var getRole = exports.getRole = function getRole(roles) {
  return (0, _utils.compose)(getRoleByName(roles), getRoleName);
};

var getRoleIfNotEmpty = exports.getRoleIfNotEmpty = function getRoleIfNotEmpty(roles) {
  return function (role) {
    return (0, _utils.composeAll)(function (_ref) {
      var _ref2 = _slicedToArray(_ref, 2),
          maybeEmpty = _ref2[0],
          isEmpty = _ref2[1];

      return !isEmpty ? maybeEmpty : role;
    }, (0, _utils.applyAll)(_utils.identity, _role.Role.isEmpty), getRole(roles))(role);
  };
};

var getOrCreateRole = exports.getOrCreateRole = function getOrCreateRole(roles) {
  return (0, _utils.compose)(getRoleIfNotEmpty(roles), _role.Role.lift);
};