'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.setRole = exports.addMemberToRole = exports.setRoleToStore = exports.setRoleToRoles = exports.updateRolesOnGrants = exports.updateRoleOnGrant = exports.deleteRoleOnGrantMembers = exports.grantHaveRole = undefined;

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _role = require('./role');

var _get = require('./get');

var _utils = require('../utils');

var grantHaveRole = exports.grantHaveRole = function grantHaveRole(role) {
  return function (grant) {
    return !!grant.members.find((0, _role.isSameRole)(role));
  };
};

var deleteRoleOnGrantMembers = exports.deleteRoleOnGrantMembers = function deleteRoleOnGrantMembers(role) {
  return function (members) {
    return (0, _utils.composeAll)(members.delete.bind(members), members.find.bind(members, (0, _role.isSameRole)(role)))(members);
  };
};

var updateRoleOnGrant = exports.updateRoleOnGrant = function updateRoleOnGrant(role) {
  return function (grant) {
    return (0, _utils.composeAll)(grant.set.bind(grant, 'members'), function (members) {
      return members.add(role);
    }, deleteRoleOnGrantMembers(role))(grant.members);
  };
};

var updateRolesOnGrants = exports.updateRolesOnGrants = function updateRolesOnGrants(store) {
  return (0, _utils.composeAll)(store.set.bind(store, 'grants'), store.grants.map.bind(store.grants), function (_ref) {
    var _ref2 = _slicedToArray(_ref, 2),
        haveRole = _ref2[0],
        updateRole = _ref2[1];

    return (0, _utils.applyIf)(updateRole)(haveRole);
  }, (0, _utils.applyAll)(grantHaveRole, updateRoleOnGrant));
};

var setRoleToRoles = exports.setRoleToRoles = function setRoleToRoles(roles) {
  return function (role) {
    return roles.set(role.name, role);
  };
};

var setRoleToStore = exports.setRoleToStore = function setRoleToStore(store) {
  return (0, _utils.compose)(store.set.bind(store, 'roles'), setRoleToRoles(store.roles));
};

var addMemberToRole = exports.addMemberToRole = function addMemberToRole(roles) {
  return function (member, role) {
    return (0, _utils.composeAll)(function (_ref3) {
      var _ref4 = _slicedToArray(_ref3, 2),
          role = _ref4[0],
          members = _ref4[1];

      return role.set('members', members);
    }, (0, _utils.applyAll)(_utils.identity, function (role) {
      return role.members.add(member);
    }), (0, _get.getRole)(roles))(role);
  };
};

var setRole = exports.setRole = function setRole(stateSetter, store) {
  return (0, _utils.composeAll)(_utils.selectLast, (0, _utils.applyAll)((0, _utils.compose)(stateSetter, function (_ref5) {
    var _ref6 = _slicedToArray(_ref5, 2),
        state = _ref6[0],
        role = _ref6[1];

    return updateRolesOnGrants(state)(role);
  }), (0, _utils.compose)(_utils.identity, _utils.selectLast)), (0, _utils.applyAll)(setRoleToStore(store), _utils.identity), addMemberToRole(store.roles));
};