'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Store = undefined;

var _immutable = require('immutable');

var _grant = require('./grant');

var _role = require('./role');

var _utils = require('./utils');

var Store = exports.Store = (0, _immutable.Record)({
  grants: (0, _immutable.Set)(),
  roles: (0, _immutable.Map)()
}, 'Store');

var setOrUniversal = function setOrUniversal(value) {
  return value.get('universal') ? (0, _utils.UniversalSet)() : (0, _immutable.Set)(value);
};

var revivers = {
  root: function root(value) {
    return new Store(value);
  },
  grants: _immutable.Set,
  roles: _immutable.Map,
  members: setOrUniversal,
  resources: setOrUniversal,
  attributes: setOrUniversal
};

Store.lift = function (store) {
  return (0, _immutable.fromJS)(store, function (key, value) {
    var reviverKey = key === '' ? 'root' : key;
    var reviver = revivers[reviverKey];

    if (reviver) {
      return reviver(value);
    } else if (value.get('name')) {
      return new _role.Role(value);
    } else {
      return new _grant.Grant(value);
    }
  });
};